@@ -25,8 +25,10 @@ class Build:
         self.table_of_contents = table_of_contents
         self.sheets = []
         self.logo_path = None
+        self.font_name = None
+        self.font_size = None
         self.index_options = {
-            'header_color': 'efefef',
+            'header_color': 'ffffff',
             'body_color': 'ffffff',
             'link_color': '2A64C5'
         }
@@ -61,6 +63,8 @@ class Build:
         try:
             wb = load_workbook(filename)
 
+            default_font = Font(name=self.font_name, size=self.font_size)
+            
             # add table of contents
             if len(self.sheets)>1:
                 offset_y = 12
@@ -92,17 +96,23 @@ class Build:
                         color_cell = wb_sheet.cell(row_range, col_range)
                         color_cell.fill = PatternFill(start_color=self.index_options['header_color'], end_color=self.index_options['header_color'], fill_type="solid")
 
+                # self.sheets tells us how many entries are in the index
                 for row_range in range(10, len(self.sheets)+100):
                     for col_range in range(1, 30):
                         color_cell = wb_sheet.cell(row_range, col_range)
                         color_cell.fill = PatternFill(start_color=self.index_options['body_color'], end_color=self.index_options['body_color'], fill_type="solid")
+                        color_cell.font = default_font
 
                 for index, sh in enumerate(self.sheets):
                     top_offset = sh.formats['offsets']['top']
                     wb_sheet = wb.worksheets[index+1]
                     wb_sheet.row_dimensions[top_offset+1].height=30
                     wb_sheet.row_dimensions[top_offset+2].height=70
-                    
+
+                    for row in wb_sheet.rows:
+                        for cell in row:
+                            cell.font = default_font                
+
                     for col_range in range(1,30):
                         wb_sheet.cell(top_offset+1,col_range).alignment = Alignment(wrap_text=True, vertical='center', horizontal='center')
                         wb_sheet.cell(top_offset+2,col_range).alignment = Alignment(wrap_text=True, vertical='center', horizontal='center')
@@ -127,8 +137,13 @@ class Build:
                     wb_sheet.cell(top_offset+1,col_range).alignment = Alignment(wrap_text=True, vertical='center', horizontal='center')
                     wb_sheet.cell(top_offset+2,col_range).alignment = Alignment(wrap_text=True, vertical='center', horizontal='center')
 
+                for row in wb_sheet.rows:
+                    for cell in row.cells:
+                        cell.font = default_font    
+
             wb.save(filename)
-        except:
+        except Exception as e:
+            print('Warning: Could not apply Excel styles. Make sure all colors are defined as ffffff rather than #ffffff.')
             pass
             
 class Sheet:
@@ -146,7 +161,9 @@ class Sheet:
                 "percentage": {},
                 "counts": {},
             },
-            "stub": {"ci":["c%"]}
+            "stub": {"ci":["c%"]},
+            "banner_border": True
+
         }
         self.set_question_format('percentage', {'text_wrap':True})
         self.set_question_format('counts', {'text_wrap':True})
@@ -189,6 +206,8 @@ class Sheet:
     def set_default_weight(self, weight):
         self.table_options['stub']['w'] = 'weight_a'
 
+    def set_banner_border(self, border):
+        self.table_options['banner_border'] = border
 
     def freeze_panes(self, row=10, column=1):
         freeze = {
@@ -222,7 +241,7 @@ class Sheet:
         old_format = self.table_options['format'][answer_type]
         self.table_options['format'][answer_type] = {**old_format, **new_format}
 
-    def set_column_format(self, answer_type, column_index, format):
+    def set_column_format_for_type(self, answer_type, column_index, format):
         new_format = {column_index: {"format":format}}
         old_format = self.table_options['format'][answer_type]
         self.table_options['format'][answer_type] = {**old_format, **new_format}
@@ -239,6 +258,7 @@ class Sheet:
         df['FORMAT'] = df['FORMAT'].apply(lambda x: json.dumps({**json.loads(x), **{'cell_format':{}}}))
         df = self.apply_sheet_options(df, self.options)
         df = self.apply_table_options(df, options)
+        df = self._append_row_to_dataframe(df)
         self.dataframes.append(df)
 
     def apply_sheet_options(self, df, options):
@@ -264,6 +284,8 @@ class Sheet:
             df = self.apply_table_format_options(df, options['format'])
         if 'row_format' in options:
             df = self.set_row_format(df, options['row_format'])
+        if 'banner_border' in options:
+            df = self.add_banner_border(df)
         return df
 
     def apply_table_format_options(self, df, options):
@@ -288,6 +310,23 @@ class Sheet:
             df.iat[row, -1] = apply_row_format(df.iloc[row, -1], format)
         return df        
 
+    def set_column_format(self, df, col_index, format):
+        def apply_column_format(x, col_index, new_format):
+            current_format = json.loads(x)
+            for i in col_index:
+                if str(i) in current_format['cell_format']:
+                    current_format['cell_format'][str(i)]['format'] = {**current_format['cell_format'][str(i)]['format'], **new_format}
+                else:
+                    current_format['cell_format'][str(i)] = {'format':new_format}
+            return json.dumps(current_format)
+        # users can send int and lists
+        if type(col_index) != list:
+            col_index = [col_index]
+        for row in list(range(df.shape[0])):
+            if 'spacer' not in json.loads(df.iloc[row, -1]):
+                df.iat[row, -1] = apply_column_format(df.iloc[row, -1], col_index, format)
+        return df     
+
     def set_format_for_type(self, df, type, format):
         """ Method to add/alter format json per cell or per question type.
 
@@ -310,11 +349,24 @@ class Sheet:
         location = df[df['FORMAT'].str.contains(f'\"type\": \"{type}\"')].index
         if len(location) == 0:
             location = df[df['FORMAT'].str.contains(f'\"original_type\": \"{type}\"')].index
-        df.at[location, ('FORMAT',)] = df.loc[location]['FORMAT'].apply(apply_format, args=[new_format])
+        loc_row_index = list(location.get_level_values(1))
+        for row in loc_row_index:
+            df.iat[row, -1] = apply_format(df.iloc[row,-1], new_format)
         df.index = old_index
         df.index = df.index.set_names(['Question', 'Values'])
         return df
 
+    def add_banner_border(self, df):
+        all_questions = df.columns.to_frame().index.to_series().reset_index()['Question']
+        # find location of first unique question (this is the far left column in each subframe)
+        all_questions = list(all_questions.drop_duplicates()[:-1].index)
+        # the question column is col 0 and we don't count that
+        if 0 in all_questions:
+            all_questions.remove(0)
+        all_questions = [i+1 for i in all_questions]
+        self.set_column_format(df, all_questions, {'left':1})
+        return df
+
     def apply_base_options(self, df, option):
         if option == 'hide':
             if 'Unweighted base' in df.index.get_level_values(1):
@@ -334,9 +386,18 @@ class Sheet:
                 df = df.drop('Unweighted base', level=1)
             if 'Base' in df.index.get_level_values(1):            
                 df = df.drop('Base', level=1)
+            bases = self._append_row_to_dataframe(bases)
             df = pd.concat([bases,df])
 
         return df
 
+    def _append_row_to_dataframe(self, df):
+        last_question = df.index.get_level_values(0)[-1]
+        mi = pd.MultiIndex.from_tuples([(last_question, ' ')])
+        empty_row_df = pd.DataFrame(columns=df.columns, data=[['']*df.shape[1]], index=mi)
+        empty_row_df.iloc[-1, -1] = '{"type":"counts", "spacer":true, "cell_format":{}}'
+        df = pd.concat([df, empty_row_df])
+        return df
+
     def combine_dataframes(self):
         return pd.concat(self.dataframes)
